@@ -20,6 +20,7 @@
 
 #define _XOPEN_SOURCE	/* strptime proto, but this hides other prototypes */
 #define _GNU_SOURCE	/* get all the other prototypes */
+#define __EXTENSIONS__	/* for solaris to get back strdup and strcasecmp */
 
 #include "config.h"
 #include "actions.h"
@@ -36,6 +37,7 @@
 
 #include <gphoto2/gphoto2-port-log.h>
 #include <gphoto2/gphoto2-setting.h>
+#include <gphoto2/gphoto2-filesys.h>
 #include <gphoto2/globals.h>
 
 #ifdef HAVE_AA
@@ -70,11 +72,11 @@ action_camera_upload_file (GPParams *p, const char *folder, const char *path)
 
 	gp_log (GP_LOG_DEBUG, "main", "Uploading file...");
 
-	CR (gp_file_new (&file));
+	CR (gp_file_new_from_fd (&file, -1));
 	res = gp_file_open (file, path);
-	if (res < 0) {
+	if (res < GP_OK) {
 		gp_file_unref (file);
-		return (res);
+		return res;
 	}
 
 	/* Check if the user specified a filename */
@@ -85,7 +87,6 @@ action_camera_upload_file (GPParams *p, const char *folder, const char *path)
 			return (res);
 		}
 	}
-
 	res = gp_camera_folder_put_file (p->camera, folder, file,
 					 p->context);
 	gp_file_unref (file);
@@ -890,7 +891,7 @@ print_version_action (GPParams __unused__ *p)
 		  "\n"
 		  "This version of gphoto2 is using the following software versions and options:\n"),
 		VERSION,
-		2006, /* year of release! */
+		2008, /* year of release! */
 		port_message
 		);
 
@@ -921,9 +922,24 @@ int
 action_camera_capture_preview (GPParams *p)
 {
 	CameraFile *file;
-	int r;
+	int	r, fd;
+        char    tmpname[20], *tmpfilename;
+
+	strcpy (tmpname, "tmpfileXXXXXX");
+	fd = mkstemp(tmpname);
+	if (fd == -1) {
+		CR (gp_file_new (&file));
+		tmpfilename = NULL;
+	} else {
+		r = gp_file_new_from_fd (&file, fd);
+		if (r < GP_OK) {
+			close (fd);
+			unlink (tmpname);
+			return r;
+		}
+		tmpfilename = tmpname;
+	}
 
-	CR (gp_file_new (&file));
 #ifdef HAVE_AA
 	r = gp_cmd_capture_preview (p->camera, file, p->context);
 #else
@@ -934,7 +950,7 @@ action_camera_capture_preview (GPParams *p)
 		return (r);
 	}
 
-	r = save_camera_file_to_file (NULL, file);
+	r = save_camera_file_to_file (NULL, file, tmpfilename);
 	gp_file_unref (file);
 	if (r < 0) 
 		return (r);
@@ -976,6 +992,95 @@ action_camera_wait_event (GPParams *p)
 	return GP_OK;
 }
 
+int
+print_storage_info (GPParams *p)
+{
+	int			ret, i, nrofsinfos;
+	CameraStorageInformation	*sinfos;
+
+	ret = gp_camera_get_storageinfo (p->camera, &sinfos, &nrofsinfos, p->context);
+	if (ret != GP_OK) {
+		if (ret == GP_ERROR_NOT_SUPPORTED)
+			printf (_("Getting storage information not supported for this camera.\n"));
+		return ret;
+	}
+	for (i=0;i<nrofsinfos;i++) {
+		printf ("[Storage %d]\n", i);
+		if (sinfos[i].fields & GP_STORAGEINFO_LABEL)
+			printf ("label=%s\n", sinfos[i].label);
+		if (sinfos[i].fields & GP_STORAGEINFO_DESCRIPTION)
+			printf ("description=%s\n", sinfos[i].description);
+		if (sinfos[i].fields & GP_STORAGEINFO_BASE)
+			printf ("basedir=%s\n", sinfos[i].basedir);
+		if (sinfos[i].fields & GP_STORAGEINFO_ACCESS) {
+			printf ("access=%d ", sinfos[i].access);
+			switch (sinfos[i].access) {
+			case GP_STORAGEINFO_AC_READWRITE:
+				printf (_("Read-Write"));
+				break;
+			case GP_STORAGEINFO_AC_READONLY:
+				printf (_("Read-Only"));
+				break;
+			case GP_STORAGEINFO_AC_READONLY_WITH_DELETE:
+				printf (_("Read-only with delete"));
+				break;
+			default:
+				printf (_("Unknown"));
+				break;
+			}
+			printf("\n");
+		}
+		if (sinfos[i].fields & GP_STORAGEINFO_STORAGETYPE) {
+			printf ("type=%d ", sinfos[i].type);
+			switch (sinfos[i].type) {
+			default:
+			case GP_STORAGEINFO_ST_UNKNOWN:
+				printf ( _("Unknown"));
+				break;
+			case GP_STORAGEINFO_ST_FIXED_ROM:
+				printf ( _("Fixed ROM"));
+				break;
+			case GP_STORAGEINFO_ST_REMOVABLE_ROM:
+				printf ( _("Removable ROM"));
+				break;
+			case GP_STORAGEINFO_ST_FIXED_RAM:
+				printf ( _("Fixed RAM"));
+				break;
+			case GP_STORAGEINFO_ST_REMOVABLE_RAM:
+				printf ( _("Removable RAM"));
+				break;
+			}
+			printf("\n");
+		}
+		if (sinfos[i].fields & GP_STORAGEINFO_FILESYSTEMTYPE) {
+			printf ("fstype=%d ", sinfos[i].type);
+			switch (sinfos[i].fstype) {
+			default:
+			case GP_STORAGEINFO_FST_UNDEFINED:
+				printf ( _("Undefined"));
+				break;
+			case GP_STORAGEINFO_FST_GENERICFLAT:
+				printf ( _("Generic Flat"));
+				break;
+			case GP_STORAGEINFO_FST_GENERICHIERARCHICAL:
+				printf ( _("Generic Hierarchical"));
+				break;
+			case GP_STORAGEINFO_FST_DCF:
+				printf ( _("Camera layout (DCIM)"));
+				break;
+			}
+			printf("\n");
+		}
+		if (sinfos[i].fields & GP_STORAGEINFO_MAXCAPACITY)
+			printf ("totalcapacity=%ld KB\n", sinfos[i].capacitykbytes);
+		if (sinfos[i].fields & GP_STORAGEINFO_FREESPACEKBYTES)
+			printf ("free=%ld KB\n", sinfos[i].freekbytes);
+		if (sinfos[i].fields & GP_STORAGEINFO_FREESPACEIMAGES)
+			printf ("freeimages=%ld\n", sinfos[i].freeimages);
+	}
+	return GP_OK;
+}
+
 int
 override_usbids_action (GPParams *p, int usb_vendor, int usb_product,
 			int usb_vendor_modified, int usb_product_modified)
@@ -1016,7 +1121,7 @@ override_usbids_action (GPParams *p, int usb_vendor, int usb_product,
 }
 
 /* time zero for debug log time stamps */
-struct timeval glob_tv_zero = { 0, 0 };
+static struct timeval glob_tv_zero = { 0, 0 };
 
 static void
 #ifdef __GNUC__
@@ -1127,11 +1232,12 @@ display_widgets (CameraWidget *widget, char *prefix) {
 	n = gp_widget_count_children (widget);
 
 	newprefix = malloc(strlen(prefix)+1+strlen(uselabel)+1);
+	if (!newprefix)
+		abort();
 	sprintf(newprefix,"%s/%s",prefix,uselabel);
 
-	if ((type != GP_WIDGET_WINDOW) && (type != GP_WIDGET_SECTION)) {
+	if ((type != GP_WIDGET_WINDOW) && (type != GP_WIDGET_SECTION))
 		printf("%s\n",newprefix);
-	}
 	for (i=0; i<n; i++) {
 		CameraWidget *child;
 	
@@ -1140,6 +1246,7 @@ display_widgets (CameraWidget *widget, char *prefix) {
 			continue;
 		display_widgets (child, newprefix);
 	}
+	free(newprefix);
 }
 
 
@@ -1214,8 +1321,9 @@ _find_widget_by_name (GPParams *p, const char *name, CameraWidget **child, Camer
  *     clean one is to add an intermediate function
  */
 
-static inline size_t
-my_strftime(char *s, size_t max, const char *fmt, const struct tm *tm) {
+static size_t
+my_strftime(char *s, size_t max, const char *fmt, const struct tm *tm)
+{
 	return strftime(s, max, fmt, tm);
 }
 
@@ -1393,18 +1501,19 @@ set_config_action (GPParams *p, const char *name, const char *value) {
 		int	t;
 
 		t = 2;
-		if (	strcasecmp (value, "off")	|| strcasecmp (value, "no")	||
-			strcasecmp (value, "false")	|| strcmp (value, "0")		||
-			strcasecmp (value, _("off"))	|| strcasecmp (value, _("no"))	||
-			strcasecmp (value, _("false"))
+		if (	!strcasecmp (value, "off")	|| !strcasecmp (value, "no")	||
+			!strcasecmp (value, "false")	|| !strcmp (value, "0")		||
+			!strcasecmp (value, _("off"))	|| !strcasecmp (value, _("no"))	||
+			!strcasecmp (value, _("false"))
 		)
 			t = 0;
-		if (	strcasecmp (value, "on")	|| strcasecmp (value, "yes")	||
-			strcasecmp (value, "true")	|| strcmp (value, "0")		||
-			strcasecmp (value, _("on"))	|| strcasecmp (value, _("yes"))	||
-			strcasecmp (value, _("true"))
+		if (	!strcasecmp (value, "on")	|| !strcasecmp (value, "yes")	||
+			!strcasecmp (value, "true")	|| !strcmp (value, "1")		||
+			!strcasecmp (value, _("on"))	|| !strcasecmp (value, _("yes"))	||
+			!strcasecmp (value, _("true"))
 		)
 			t = 1;
+		/*fprintf (stderr," value %s, t %d\n", value, t);*/
 		if (t == 2) {
 			gp_context_error (p->context, _("The passed value %s is not a valid toggle value."), value);
 			ret = GP_ERROR_BAD_PARAMETERS;
